﻿var scriptName = 'Import clips - ClipExporter';
app.beginUndoGroup(scriptName);

var proj = (!app.project) ? app.newProject() : app.project; // if no project is open, create a new project

// RESOURCES
[[--resources--]]


[[--audio--]]
[[--refmcvideo--]]


// FUNCTIONS

function checkAndGetName(theName,type){
    // checks if the given name exists. if true, autoincrement and return incremented name
    var projItemCount = app.project.numItems;
    var counter = 0;
    for (var i = 1; i <= projItemCount ; i++) if (proj.item(i) instanceof type && proj.item(i).name.indexOf(theName) != -1 )     counter++;
    return (counter == 0) ? theName : theName +' '+counter;
}
function setRetimePointsOnLayer(layer,s){
    // first enable time remaping
    layer.timeRemapEnabled = true;
    var maxKeyValue = layer.timeRemap.keyValue(2);
    // remove keyframe that automatically has been set at the end of the clip because it messes up remaping
    layer.timeRemap.removeKey(2);
    
    for(var i = 0; i < s.retime.keys.length; i++){
        var key = s.retime.keys[i];
        layer.timeRemap.setValueAtTime(key.time, (key.value > maxKeyValue ? maxKeyValue : key.value));
    }
}
function timeCompensate(item, val){
    // compensates timecode
    if (item.retime) return val - item.retime.startSubtraTime;
    else if(item.type == 'title') return (val > 3000) ? (val - s.start) : val;
    else return val - resources[item.ref].start;
}
function setupLayer(layer,s){
    // START TIME - needs to be setup before anything else
    if(s.retime){
        setRetimePointsOnLayer(layer,s);
        layer.inPoint = s.start - s.retime.startSubtraTime;
    }
    else if (s.type == 'title') layer.inPoint = s.start - 3600;
    else layer.inPoint = s.start - resRef.start;
    
    if (s.handles) layer.inPoint += s.handles[0];
    
    // ENABLE
    if(s.isAudioClip){
        layer.enabled = false;
        layer.audioEnabled = s.enabled;
    }
    else layer.enabled = s.enabled;
    if(s.srcEnable == 'video') layer.audioEnabled = false;
    var parentComp = layer.containingComp;
    if(s.type == 'video' && s.audioEnabled != undefined) layer.audioEnabled = s.audioEnabled;
    
    // BLENDING
    if (s.blendMode) layer.blendingMode = s.blendMode;
    if (s.bAmount) {
        if (isNaN(s.bAmount)) for(var b = 0; b < s.bAmount.length;b++) {
            layer.opacity.setValueAtTime(timeCompensate(s, s.bAmount[b].time), s.bAmount[b].value*100);
        } else if(typeof s.bAmount == "number") layer.opacity.setValue(s.bAmount*100);
    }
    
    // ADJUST CONFORM
    if (s.type != 'audio'){        
        // ratio = ratio of the source footage (item) to the compound clips where it is nested in
        var ratio = (layer.width / layer.height) - ((parentComp.width * parentComp.pixelAspect) / parentComp.height);
        wRatio = (parentComp.width / layer.width)*100;
        hRatio = (parentComp.height / layer.height)*100;
        if(s.conform == 'none') layer.scale.setValue([100, 100 ]);
        if (ratio <= 0) {
            if(s.conform == 'fill') layer.scale.setValue([wRatio, wRatio]);
            if(s.conform == 'fit') layer.scale.setValue([hRatio, hRatio]);
        } else {
            if(s.conformm == 'fill') layer.scale.setValue([hRatio, hRatio]);
            if(s.conform == 'fit') layer.scale.setValue([wRatio, wRatio]);
        }
    }
    // TRANSFORM
    if (s.tEnabled){
        
        // POSITION
        if(s.tPosition){
            // tPosition = [123,123] -> non-animated
            // tPosition = [{time:123,value:1254.00,axis:'X'},{...}] ->animated
            var initValueP = layer.position.value;
            if (isNaN(s.tPosition[0])){
                // animated property
                layer.position.dimensionsSeparated = true; // separate dimensions
                for(var t=0; t < s.tPosition.length; t++){
                    var dimIndex = (s.tPosition[t].axis == "X") ? 0 : 1;
                    var axis = layer.position.getSeparationFollower(dimIndex);
                    var keyTime = timeCompensate(s,s.tPosition[t].time);
                    var currentValueP = axis.valueAtTime(keyTime,true);
                    var dividend = (s.tPosition[t].axis == "X") ? 100 : -100;
                    var posValue = initValueP[dimIndex] + (parentComp.height * (s.tPosition[t].value/dividend))
                    axis.setValueAtTime(keyTime , posValue);
                }
            }
            else {
                // static property
                var posValue = [(initValueP[0] + (parentComp.height * (s.tPosition[0]/100))) , (initValueP[1] + (parentComp.height * (s.tPosition[1]/-100))) ,0];
                layer.position.setValue(posValue);
            }
        }
        // ANCHOR POINT
        if(s.tAnchor){
            var hRatio = (parentComp.height / layer.height);
            function convertPercentValuesA(aValue,initAnch,currAnch,axis){
                if (axis == "X") return [initAnch[0] + ((aValue / 100) * parentComp.height / hRatio), currAnch[1]];
                if (axis == "Y") return [currAnch[0], initAnch[1] + ((aValue / -100) * parentComp.height / hRatio) ];
                var xAnch = (aValue[0] / 100) * parentComp.height / hRatio;
                var yAnch = (aValue[1] / -100) * parentComp.height / hRatio;
                return [xAnch+initAnch[0] , yAnch+initAnch[1]];
            }
            
            var initValueA = layer.anchorPoint.value;
            if (isNaN(s.tAnchor[0])){
                for(var t=0; t < s.tAnchor.length; t++){
                    var keyTime = timeCompensate(s,s.tAnchor[t].time);
                    var currentValueA = layer.anchorPoint.valueAtTime(keyTime,true);
                    layer.anchorPoint.setValueAtTime(keyTime,convertPercentValuesA(s.tAnchor[t].value,initValueA,currentValueA,s.tAnchor[t].axis));
                }
            }
            else layer.anchorPoint.setValue(convertPercentValuesA(s.tAnchor,initValueA,100,''));
        }
        // SCALE
        if(s.tScale){
            function convertPercentValuesS(sValue,initLayScale,curScale,axis){
                if (axis == "X") return [initLayScale[0]*sValue , curScale[1]];
                if (axis == "Y") return [curScale[0], initLayScale[1]*sValue ];
                return [initLayScale[0]*sValue[0] , initLayScale[1]*sValue[1]];
            }
            var initLayScale = layer.scale.value;
            if (isNaN(s.tScale[0])){
                for(var t=0; t < s.tScale.length; t++){
                    var keyTime = timeCompensate(s,s.tScale[t].time);
                    var currentScale = layer.scale.valueAtTime(keyTime,true);
                    layer.scale.setValueAtTime(keyTime,convertPercentValuesS(s.tScale[t].value,initLayScale,currentScale,s.tScale[t].axis));
                    
                }
            }
            else layer.scale.setValue(convertPercentValuesS(s.tScale,layer.scale.value,100,''));
        }
        // ROTATION
        if(s.tRotation){
            if (isNaN(s.tRotation)){
                for(var t=0; t < s.tRotation.length; t++)
                    layer.rotation.setValueAtTime(timeCompensate(s,s.tRotation[t].time),(s.tRotation[t].value*-1));
            }
            else layer.rotation.setValue(s.tRotation*-1);
        }
    }
    // CORNER PIN
    if (s.cEnabled){
        function convertPercentValuesC(cValue,initPin,curPinPos,axis){
            if (axis == "X") return [(initPin[0] + (parentComp.height * (cValue/100))) , curPinPos[1] ];
            if (axis == "Y") return [curPinPos[0] , (initPin[1] + (parentComp.height * (cValue/-100))) ];
            else return [(initPin[0] + (parentComp.height * (cValue[0]/100))) , (initPin[1] + (parentComp.height * (cValue[1]/-100)))];
        }
        function setPin(pinFCP, pinAE){
            var initPinPos = pinAE.value;
            if (isNaN(pinFCP[0])){
                for(var t=0; t < pinFCP.length; t++){
                    var keyTime = timeCompensate(s,pinFCP[t].time);
                    var currentPinPos = pinAE.valueAtTime(keyTime,true);
                    var theValue = convertPercentValuesC(pinFCP[t].value,initPinPos,currentPinPos,pinFCP[t].axis);
                    pinAE.setValueAtTime(keyTime,theValue);
                }
            }
            else pinAE.setValue(convertPercentValuesC(pinFCP,initPinPos,'',''));
        }
        var cPin = layer.Effects.addProperty("ADBE Corner Pin");
        if (s.cTopLeft)  setPin(s.cTopLeft,cPin.property(1));
        if (s.cTopRight) setPin(s.cTopRight,cPin.property(2));
        if (s.cBotLeft)  setPin(s.cBotLeft,cPin.property(3));
        if (s.cBotRight) setPin(s.cBotRight,cPin.property(4));
    }
    // AUDIO VOLUME
    if (s.tVolume && layer.audioLevels.elided){
        if (isNaN(parseInt(s.tVolume[0]))){
            for(var t=0; t < s.tVolume.length; t++){
                var keyTime = timeCompensate(s,s.tVolume[t].time);
                layer.audioLevels.setValueAtTime(keyTime,[s.tVolume[t].value,s.tVolume[t].value]);
                
            }
        }
        else layer.audioLevels.setValue([s.tVolume[0],s.tVolume[0]]);
    }
    
    // NOTES
    if (s.note) layer.comment = layer.comment + s.note;
    
    // MARKER
    if(s.marker){
        for(var m = 0; m < s.marker.length;m++) {
            layer.marker.setValueAtTime(timeCompensate(s,s.marker[m].time), new MarkerValue(s.marker[m].value));
        }
    }
    
    // TIMINGS
    layer.startTime = s.offset - layer.inPoint;
    var outP = layer.inPoint + s.dur;
    if (s.handles) outP = outP - s.handles[1] - s.handles[0];
    layer.outPoint = outP > 10700 ? 10700 : outP;
    
}
// FUNCTIONS END

// create a folders
var mainF = proj.items.addFolder(checkAndGetName(resources.main.name,FolderItem));
var filesF = proj.items.addFolder(checkAndGetName('Files',FolderItem));
var compsF = proj.items.addFolder(checkAndGetName('Compound Clips',FolderItem));
var clipsF = proj.items.addFolder(checkAndGetName('Clips',FolderItem));
filesF.parentFolder = mainF;
compsF.parentFolder = mainF;
clipsF.parentFolder = mainF;

// SETUP RESOURCES - asset-, media- and clip-objects
var missingFiles = [];
for(var key in resources){
    r = resources[key];
    
    // import source files -> Files folder
    if (r.type == 'asset'){
        var srcfile = new File(r.src);
        if (srcfile.exists){
            try{
                var footage = proj.importFile(new ImportOptions(srcfile));
                r.resItem = footage;
                footage.parentFolder = filesF;
            } catch(e){ }
        } else {
            missingFiles.push(r.src);
        }
    }
    // create comps and multicams -> Compound Clips folder
    if (r.type == 'media' || r.type == 'clip'){
        var compName = (key == 'main') ? '<< '+r.name+' >>' : r.name;
        var comp = proj.items.addComp(checkAndGetName(compName,CompItem),r.w,r.h,r.pasp,(r.dur > 0.0 ? r.dur : 10000),r.fps);
        if (r.retime) {
            comp.displayStartTime = r.retime.startSubtraValue;
            comp.duration = r.retime.fullUnretimedDuration;
        }
        else comp.displayStartTime = r.start + 0.007;
        
        if (r.type == 'clip'){
            if (r.isAudioClip) comp.label = 7; //sea foam
            else comp.label = 9; //green
            comp.parentFolder = clipsF;
            r.ref = key; // add ref property to object. needed for later iteration
            rmv.push(r);
        }
        if (r.type == 'media') {
            comp.label = (r.isMC) ? 8 : 11; // blue or orange
            comp.parentFolder = (key == 'main') ? mainF :compsF;
            
        }
        if (r.note) comp.comment = r.note;
        r.resItem = comp; // add compItem to the resources object
    }
}
// sort array by object property "lane"
rmv.sort( function(a,b) {
                if (a.lane != b.lane) return a.lane - b.lane;
                else return a.offset - b.offset;
                });

// MULTICAMS
// first setup all multicams with their content (angles)
var mcLengths = {};
for (var i = 0; i < rmv.length; i++) {
    
    if (! rmv[i].angleID) continue; // discard non-multicam angle items
    
    var s = rmv[i];
    
    // get referenced asset or media objects
    var resRef = resources[s.ref];
    var pID = (s.parentClipID) ? s.parentClipID : s.parent;
    var parentComp;
    if (resources[pID]) parentComp = resources[pID];
    else continue; // discard clips without parents: non active auditions
    if(!parentComp.resItem || !resRef) continue;
    
    // if footage is missing, create placeholder
    if(!resRef.resItem) {
        if(! s.fps > 0.0 || ! s.fps) s.fps = resources['main'].fps;
        resRef.resItem = proj.importPlaceholder(resRef.name, s.w, s.h, s.fps, resRef.dur);
        resRef.resItem.parentFolder = filesF;
    }
    
    // add footage item to the multicam comp
    var theLayer = parentComp.resItem.layers.add(resRef.resItem);
    
    
    setupLayer(theLayer,s);
    theLayer.comment = theLayer.comment + 'angleID'+s.angleID+'angleID';
    theLayer.name = s.angleName;
    
    // set the duration based on the longest clip in the multicam. FCP X doesn´t give any duration values for multicam
    if ((theLayer.startTime + s.dur) > mcLengths[s.angleID]) mcLengths[s.angleID] = theLayer.startTime + s.dur;
    parentComp.resItem.duration = mcLengths[s.angleID];
    
}
// AUDIO
// put audio files in comps first
for (var i = 0; i < au.length; i++){
    var s = au[i];
    if (s.angleID) continue; // discard multicam angles
    // get referenced asset or media
    var resRef = resources[s.ref];
    var pID = (s.parentClipID) ? s.parentClipID : s.parent;
    var parentComp;
    if (resources[pID]) parentComp = resources[pID];
    else continue; // discard clips without parents: non active auditions
    if(!resRef.resItem) continue;
    if(!parentComp.resItem) continue;
    var theLayer = parentComp.resItem.layers.add(resRef.resItem);
    theLayer.name = s.name;
    setupLayer(theLayer,s);
    
}

// CLIPS AND COMPS
// create a layer for comps or footage in their parent´s comp
for (var i = 0; i < rmv.length; i++) {
    var s = rmv[i];
    if (s.angleID) continue;// discard multicam angles
    
    // get referenced asset or media
    var resRef = resources[s.ref];
    var pID = (s.parentClipID) ? s.parentClipID : s.parent;
    var parentComp;
    if (resources[pID]) parentComp = resources[pID];
    else continue; // discard clips without parents: non active auditions
    
    // if footage is missing, create placeholder
    if(s.type != 'title' && !resRef.resItem) {
        if(! s.fps > 0.0 || ! s.fps) s.fps = resources['main'].fps;
        var placeholderName = resRef.name;
        if (s.type =='img' || s.type =='video')
            placeholderName = resRef.name.substring(resRef.name.lastIndexOf('/')+1);;
        resRef.resItem = proj.importPlaceholder(placeholderName, s.w, s.h, s.fps, resRef.dur);
        resRef.resItem.parentFolder = filesF;
    }
    
    // create layer (comp or footage)
    if(!parentComp.resItem) continue;
    var theLayer;
    if(s.type == 'title'){
        theLayer = parentComp.resItem.layers.addText(s.text);
        theLayer.name = s.name;
        setupLayer(theLayer,s);
        continue;
    }

    if(s.type == 'mc'){
        //duplicate the main reference multicam comp
        var resRefMc = resRef.resItem.duplicate();
        theLayer = parentComp.resItem.layers.add(resRefMc);
        // resRefMc, make only layers visible with s.activeAngle in their name
        var resRefMcLayers = resRefMc.layers;
        // disable inactive angle layers in the multicam comp
        for(var l = 1; l < resRefMcLayers.length+1; l++){
            // extract the angleID from the comments
            var comment = resRefMcLayers[l].comment;
            var layerAngle = comment.substring(comment.indexOf("angleID")+7,comment.lastIndexOf("angleID"));
            if (layerAngle != s.activeAngle) resRefMcLayers[l].enabled = false;
            // remove the angleID from the comments
            resRefMcLayers[l].comment = comment.replace(/angleID(.*?)angleID/,'');
        }
    }
    else if (s.precompose){
        var comp = proj.items.addComp(checkAndGetName(s.name,CompItem),s.w,s.h,s.pasp,(s.dur > 0.0 ? s.dur : 10000),s.fps);
        comp.displayStartTime = s.start + 0.007;
        comp.label = 9; //green
        comp.parentFolder = clipsF;
        comp.layers.add(resRef.resItem);
        theLayer = parentComp.resItem.layers.add(comp);
    }
    else theLayer = parentComp.resItem.layers.add(resRef.resItem);
    
    
    theLayer.name = s.name;
    setupLayer(theLayer,s);
}

// remove empty and unused footage or comps
for(var i = 1; i<=filesF.numItems; i++){
    if(filesF.item(i).usedIn.length == 0) filesF.item(i).remove();
}
function traverseBackwardsCheck(resourceItem){
    if (resourceItem.resItem){
        if (resourceItem.resItem.numLayers == 0) {
            resourceItem.resItem.remove();
            resourceItem.resItem = null;
        }else if (resourceItem.resItem.usedIn.length == 0) {
            resourceItem.resItem.remove();
            resourceItem.resItem = null;
        }
        if (resourceItem.parentClipID) traverseBackwardsCheck( resources[resourceItem.parentClipID]);
    }
}

for(var key in resources){
    if (resources[key].type == 'asset') continue;
    if (key == 'main') continue;
    traverseBackwardsCheck(resources[key]);
}

if(missingFiles.length > 0){
    //alert('Import complete but missing files:\n'+missingFiles.toString().replace(",","\n"),'ClipExporter import');
    alert('Import complete but missing files.\nPlaceholders were added.','ClipExporter import');
} else{
    alert('Project import complete.','ClipExporter import');
}
app.endUndoGroup();